/*
    Copyright 2009 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#else
    #define stristr strcasestr
#endif

typedef unsigned char   u8;
typedef unsigned int    u32;



#define VER     "0.1"



int hex2byte(u8 *str) {
    int     i,
            n;
    u8      *p;

    p = str;
    for(i = 0; *p; i++, p += 2) {
        if(sscanf(p, "%02X", &n) != 1) break;
        str[i] = n;
    }
    return(i);
}



u32 myrandn = 0;

void mysrand(int num) {
    myrandn = num;
}

int myrand(u32 max) {
    myrandn = (myrandn * 0x8088405) + 1;
    return(((long long)myrandn * max) >> 32);   // blah
}



u32 rol8(u32 n1, u32 n2) {
    return((n1 << n2) | (n1 >> (8 - n2)));
}



void tc_decrypt(u8 *pwd) {  // rose encryption or what?
    int     i,
            a,
            b,
            len;
    u8      t;

    len = hex2byte(pwd);
    len -= 4;
    if(len < 0) return;

    mysrand(849521);
    for(i = 0; i < len; i++) {
        pwd[i] = rol8(pwd[i], myrand(8));
    }

    mysrand(12345);
    for(i = 0; i < 256; i++) {
        a = myrand(len);
        b = myrand(len);
        t = pwd[a];
        pwd[a] = pwd[b];
        pwd[b] = t;
    }

    mysrand(42340);
    for(i = 0; i < len; i++) {
        pwd[i] ^= myrand(256);
    }

    mysrand(54321);
    for(i = 0; i < len; i++) {
        pwd[i] -= myrand(256);
    }

    pwd[len] = 0;
}



void wait_exit(int ret) {
    fprintf(stderr, "\npress RETURN to exit\n");
    fgetc(stdin);
    exit(ret);
}



void std_err(void) {
    perror("\nError");
    wait_exit(1);
}



int main(int argc, char *argv[]) {
    FILE    *fd;
    u8      buff[1024],
            *fname,
            *p;

    fputs("\n"
        "Total Commander password decrypter "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <wcx_ftp.ini/password>\n"
            "\n", argv[0]);
        wait_exit(1);
    }
    fname = argv[1];

    fd = fopen(fname, "rb");
    if(!fd) {
        p = strdup(fname);  // not needed
        tc_decrypt(p);
        printf("\n  PASSWORD: %s\n", p);
    } else {
        while(fgets(buff, sizeof(buff), fd)) {
            for(p = buff; *p && (*p != '\r') && (*p != '\n'); p++);
            *p = 0;

            p = strchr(buff, '=');
            if(!p) continue;
            *p++ = 0;
            if(stristr(buff, "password")) {
                tc_decrypt(p);
                printf("  %-12s %s\n", buff, p);
            }
        }
        fclose(fd);
    }

    wait_exit(0);
    return(0);
}


